@@ -1,9 +1,9 @@
 (*
    PostgreSQL-OCAML - OCaml-interface to the PostgreSQL database
 
-   Copyright (C) 2004  Markus Mottl
-   email: markus@oefai.at
-   WWW:   http://www.oefai.at/~markus
+   Copyright (C) 2004-  Markus Mottl
+   email: markus.mottl@gmail.com
+   WWW:   http://www.ocaml.info
 
    Copyright (C) 2001  Alain Frisch  (version: postgres-20010808)
    email: Alain.Frisch@ens.fr
@@ -24,7 +24,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
-(* $Id: postgresql.ml,v 1.5 2004/08/05 19:00:55 mottl Exp $ *)
+(* $Id: postgresql.ml,v 1.7 2006/01/24 21:11:09 mottl Exp $ *)
 
 open Printf
 
@@ -36,6 +36,12 @@ exception InternalError of string
 
 let invalid_oid = 0
 
+module FFormat = struct
+  type t =
+    | TEXT
+    | BINARY
+end
+
 type ftype =
   | BOOL
   | BYTEA
@@ -330,6 +336,7 @@ module Stub = struct
   external nfields : result -> int = "PQnfields_stub" "noalloc"
   external fname : result -> int -> string = "PQfname_stub"
   external fnumber : result -> string -> int ="PQfnumber_stub" "noalloc"
+  external fformat : result -> int -> FFormat.t = "PQfformat_stub" "noalloc"
   external ftype : result -> int -> oid = "PQftype_stub" "noalloc"
   external fmod : result -> int -> int = "PQfmod_stub" "noalloc"
   external fsize : result -> int -> int = "PQfsize_stub" "noalloc"
@@ -477,6 +484,10 @@ object
     let n = Stub.fnumber res s in
     if n = -1 then raise Not_found else n
 
+  method fformat field =
+    check_field field;
+    Stub.fformat res field
+
   method ftype field =
     check_field field;
     ftype_of_oid (Stub.ftype res field)
@@ -485,8 +496,8 @@ object
   method fsize field = check_field field; Stub.fsize res field
 
   method getvalue tuple field =
-    check_field field; check_tuple tuple;
-    if binary_tuples then raise (Error Binary);
+    check_field field;
+    check_tuple tuple;
     Stub.getvalue res tuple field
 
   method getisnull tuple field =
