@@ -1,9 +1,9 @@
 /*
    PostgreSQL-OCAML - OCaml-interface to the PostgreSQL database
 
-   Copyright (C) 2004  Markus Mottl
-   email: markus@oefai.at
-   WWW:   http://www.oefai.at/~markus
+   Copyright (C) 2004-  Markus Mottl
+   email: markus.mottl@gmail.com
+   WWW:   http://www.ocaml.info
 
    Copyright (C) 2001  Alain Frisch  (version: postgres-20010808)
    email: Alain.Frisch@ens.fr
@@ -24,7 +24,7 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* $Id: postgresql_stubs.c,v 1.12 2005/05/31 16:00:47 mottl Exp $ */
+/* $Id: postgresql_stubs.c,v 1.15 2006/01/27 22:59:34 mottl Exp $ */
 
 #include <string.h>
 
@@ -105,7 +105,7 @@ static value none = Val_int(0);
 static value make_some(value v)
 {
   CAMLparam1(v);
-  value res = alloc_small(1, 0);
+  value res = caml_alloc_small(1, 0);
   Field(res, 0) = v;
   CAMLreturn(res);
 }
@@ -117,7 +117,7 @@ static value *exc_InternalError = NULL;  /* Exception [InternalError] */
 CAMLprim value PQocaml_init(value unit)
 {
   register_global_root(&empty_string);
-  empty_string = alloc_string(0);
+  empty_string = caml_alloc_string(0);
   exc_Oid = caml_named_value("Postgresql.Oid");
   exc_InternalError = caml_named_value("Postgresql.InternalError");
   return Val_unit;
@@ -231,8 +231,8 @@ CAMLprim value PQconnectdb_stub(value vconn_info)
   caml_leave_blocking_section();
 
   /* One may raise this 30 to 500 for instance if the program takes
-     responsability of closing connections */
-  vconn = alloc_final(3, free_conn, 1, 30);
+     responsibility of closing connections */
+  vconn = caml_alloc_final(3, free_conn, 1, 30);
 
   set_conn(vconn, conn);
   set_conn_cb(vconn, NULL);
@@ -262,41 +262,42 @@ CAMLprim value PQconndefaults_stub(value unit)
   PQconninfoOption *cios = PQconndefaults(), *p = cios;
   int i, j, n;
 
-  while(p->keyword != NULL) p++;
+  while (p->keyword != NULL) p++;
 
   n = p - cios;
-  res = alloc_tuple(n);
+  res = caml_alloc_tuple(n);
 
   for (i = 0; i < n; i++, cios++) {
-    value el = alloc_small(7, 0);
+    value el = caml_alloc_small(7, 0);
     for (j = 0; j < 7; j++) { Field(el, j) = none; };
     Store_field(res, i, el);
-    Field(el, 0) = copy_string(cios->keyword);
-    modify(&Field(el, 1), copy_string(cios->envvar));
+    Field(el, 0) = caml_copy_string(cios->keyword);
+    modify(&Field(el, 1), caml_copy_string(cios->envvar));
     if (cios->compiled) {
-      value some = make_some(copy_string(cios->compiled));
+      value some = make_some(caml_copy_string(cios->compiled));
       modify(&Field(el, 2), some);
     };
     if (cios->val) {
-      value some = make_some(copy_string(cios->val));
+      value some = make_some(caml_copy_string(cios->val));
       modify(&Field(el, 3), some);
     };
-    modify(&Field(el, 4), copy_string(cios->label));
-    modify(&Field(el, 5), copy_string(cios->dispchar));
+    modify(&Field(el, 4), caml_copy_string(cios->label));
+    modify(&Field(el, 5), caml_copy_string(cios->dispchar));
     modify(&Field(el, 6), Val_int(cios->dispsize));
   };
 
   CAMLreturn(res);
 }
 
-static value make_string(const char *s)
-{ return (s ? copy_string(s) : empty_string); }
+static inline value make_string(const char *s)
+{ return (s ? caml_copy_string(s) : empty_string); }
 
 #define conn_info(fun, ret) \
   CAMLprim value fun##_stub(value vconn) \
   { \
     CAMLparam1(vconn); \
-    CAMLreturn(ret(fun(get_conn(vconn)))); \
+    value v_ret = ret(fun(get_conn(vconn))); \
+    CAMLreturn(v_ret); \
   }
 
 #define noalloc_conn_info(fun, ret) \
@@ -325,7 +326,8 @@ noalloc_conn_info(PQbackendPID, Val_int)
 #define res_info(fun, ret) \
   CAMLprim value fun##_stub(value vres) { \
     CAMLparam1(vres); \
-    CAMLreturn(ret(fun(get_res(vres)))); \
+    value v_ret = ret(fun(get_res(vres))); \
+    CAMLreturn(v_ret); \
   }
 
 #define noalloc_res_info(fun, ret) \
@@ -335,7 +337,8 @@ noalloc_conn_info(PQbackendPID, Val_int)
   CAMLprim value fun##_stub(value vres, value field_num) \
   { \
     CAMLparam1(vres); \
-    CAMLreturn(ret(fun(get_res(vres), Int_val(field_num)))); \
+    value v_ret = ret(fun(get_res(vres), Int_val(field_num))); \
+    CAMLreturn(v_ret); \
   }
 
 #define noalloc_fieldnum_info(fun, ret) \
@@ -348,7 +351,9 @@ noalloc_conn_info(PQbackendPID, Val_int)
   CAMLprim value fun##_stub(value vres, value tup_num, value field_num) \
   { \
     CAMLparam1(vres); \
-    CAMLreturn(ret(fun(get_res(vres), Int_val(tup_num), Int_val(field_num)))); \
+    value v_ret = \
+      ret(fun(get_res(vres), Int_val(tup_num), Int_val(field_num))); \
+    CAMLreturn(v_ret); \
   }
 
 #define noalloc_field_info(fun, ret) \
@@ -367,13 +372,14 @@ static void free_result(value vres)
   set_res(vres, NULL);
 }
 
-CAMLprim value PQres_isnull(value vres) {
+CAMLprim value PQres_isnull(value vres)
+{
   return Val_int((get_res(vres)) ? 0 : 1);
 }
 
 static value alloc_result(PGresult *res, np_callback *cb)
 {
-  value vres = alloc_final(3, free_result, 1, 500);
+  value vres = caml_alloc_final(3, free_result, 1, 500);
   set_res(vres, res);
   set_res_cb(vres, cb);
   np_incr_refcount(cb);
@@ -406,12 +412,29 @@ CAMLprim value PQfnumber_stub(value vres, value field_name)
   return Val_int(PQfnumber(get_res(vres), String_val(field_name)));
 }
 
+noalloc_fieldnum_info(PQfformat, Val_int)
 noalloc_fieldnum_info(PQftype, Val_int)
 noalloc_fieldnum_info(PQfsize, Val_int)
 noalloc_fieldnum_info(PQfmod, Val_int)
 
-/* Not for binary result ! */
-field_info(PQgetvalue, make_string)
+CAMLprim value PQgetvalue_stub(value vres, value v_tup_num, value v_field_num)
+{
+  CAMLparam1(vres);
+  value v_str;
+  PGresult *res = get_res(vres);
+  int field_num = Int_val(v_field_num);
+  int tup_num = Int_val(v_tup_num);
+  char *str = PQgetvalue(res, tup_num, field_num);
+  if (PQfformat(res, field_num) == 0) v_str = make_string(str);
+  else {
+    /* Assume binary format! */
+    int len = PQgetlength(res, tup_num, field_num);
+    v_str = len ? empty_string : caml_alloc_string(len);
+    memcpy(String_val(v_str), str, len);
+  }
+  CAMLreturn(v_str);
+}
+
 noalloc_field_info(PQgetlength, Val_int)
 noalloc_field_info(PQgetisnull, Val_int)
 
@@ -422,8 +445,10 @@ noalloc_res_info(PQoidValue, Val_int)
 CAMLprim value PQmakeEmptyPGresult_stub(value vconn, value status)
 {
   CAMLparam1(vconn);
-  CAMLreturn(alloc_result(PQmakeEmptyPGresult(get_conn(vconn), Int_val(status)),
-                          get_conn_cb(vconn)));
+  value v_res =
+    alloc_result(PQmakeEmptyPGresult(get_conn(vconn), Int_val(status)),
+                 get_conn_cb(vconn));
+  CAMLreturn(v_res);
 }
 
 
@@ -444,7 +469,8 @@ CAMLprim value PQsendQuery_stub(value vconn, value query)
 CAMLprim value PQgetResult_stub(value vconn)
 {
   CAMLparam1(vconn);
-  CAMLreturn(alloc_result(PQgetResult(get_conn(vconn)), get_conn_cb(vconn)));
+  value v_res = alloc_result(PQgetResult(get_conn(vconn)), get_conn_cb(vconn));
+  CAMLreturn(v_res);
 }
 
 noalloc_conn_info(PQconsumeInput, Val_int)
@@ -468,7 +494,7 @@ CAMLprim value PQescapeBytea_stub(value v_from, value v_pos_from, value v_len)
     (char *) PQescapeBytea(
       (unsigned char *) String_val(v_from) + Int_val(v_pos_from),
       Int_val(v_len), &len);
-  value v_res = alloc_string(len - 1);
+  value v_res = caml_alloc_string(len - 1);
   memcpy(String_val(v_res), buf, len);
   free(buf);
   return v_res;
@@ -482,7 +508,7 @@ CAMLprim value PQunescapeBytea_stub(value v_from)
   if (buf == NULL)
     failwith("Postgresql.unescape_bytea: illegal bytea string");
   else {
-    value v_res = alloc_string(len);
+    value v_res = caml_alloc_string(len);
     memcpy(String_val(v_res), buf, len);
     free(buf);
     return v_res;
@@ -501,7 +527,7 @@ CAMLprim value PQnotifies_stub(value vconn)
     CAMLlocal1(str);
     value couple;
     str = make_string(noti->relname);
-    couple = alloc_small(2, 0);
+    couple = caml_alloc_small(2, 0);
     Field(couple, 0) = str;
     Field(couple, 1) = Val_int(noti->be_pid);
     CAMLreturn(make_some(couple));
